/*
 * distancetask.c
 *
 *  Created on: 1 may. 2018
 *  Author: Antonio Pezuela García
 */



#include "distancetask.h"

/* Driver Header files */
#include <ti/drivers/Timer.h>

/* Queues in use */
#include <Queues/queuedebug.h>
#include <Queues/queueroverstatus.h>
#include <Watchdog/watchdog.h>

/* RTOS headers */
#include <FreeRTOS.h>
#include <semphr.h>
#include <task.h>

#include "Board.h"


/* Local functions headers */

//*****************************************************************************
//!
//! \brief This timer callback set the end of trigger pulse
//!
//! \param  xTimerHandle,   handle to the originating timer (trigger timer).
//!
//! \return none
//!
//*****************************************************************************
void prvTriggerTimerCallback(Timer_Handle xTimerHandle);

//*****************************************************************************
//!
//! \brief This interruption callback is is use when system detects a transition in echo port
//!
//!     Transition from LOW to HIGH means the start of echo pulse
//!     Transition from HIHG to LOW means the end of echo pulse before timeout
//!
//! \param  myHandle,
//!
//! \return none
//!
//*****************************************************************************
void prvEchoPinIntCallBack(uint_least8_t index);

//*****************************************************************************
//!
//! \brief This timer callback is is use when ECHO port is high value more than timeout configured
//!        this means obstacle distance is higher than configured minimum distance
//!         it will unlock echo timeout semaphore and distable echo interruption
//!
//! \param  xTimerHandle, handle to originating timer
//!
//! \return none
//!
//*****************************************************************************
void prvEchoTimerCallback(Timer_Handle xTimerHandle);

//*****************************************************************************
//!
//! \brief This routine active trigger pulse to start HC-SR04 distance measuring
//!         it sets a HIGH value in trigger GPIO port [Board_TRIGGER_ECHO]
//!
//! \param  myHandle,
//!
//! \return none
//!
//*****************************************************************************
void prvLaunchTriggerPulse(Timer_Handle triggerTimer);


//*****************************************************************************
//!
//! \brief  Check distance to the obstacle in front of the sensor
//!
//! \param  triggertimer,  timer to measure trigger pulse width
//! \param  echoTimer,     timer for echo input timeout
//! \param  taskHandle,    routine will send a notification to this task
//!                        if there is a obstacle in the road
//! \return none
//!
//*****************************************************************************
void prvCheckDistance(Timer_Handle triggetimer, Timer_Handle echoTimer, TaskHandle_t taskHandle);

/* Global file variables */

/* Currently System only use one sensor (Front activeSensor==0) */
/* In the future it will let active another sensors sharing a echo port */
uint8_t activeSensor;

/*
 * This semaphore allows start Echo timer
 */
SemaphoreHandle_t echoSemaphore;

/* This semaphore is unlock if timeout echo input timeout passed  */
SemaphoreHandle_t echoTimeoutSemaphore;

//*****************************************************************************
//!
//! \brief  Distance task.
//! This task is responsible of manage ultrasonic sensors HC-SR04 to keep rover
//!
//!
//! MSP432P401R GPIO PORTS  in use:
//!     Front sensor:
//!         HC-SR04 TRIGGER -> PORT P4.4    [Board_GPIO_TRIGGER_FRONT]
//!         HC-SR04 ECHO    -> PORT P4.0    [Board_GPIO_ECHO]
//!
//!     Back sensor [NOT IN USE]:
//!         HC-SR04 TRIGGER -> PORT
//!         HC-SR04 ECHO    -> PORT P4.0    [Board_GPIO_ECHO]
//!
//! \param  none
//!
//! \return none
//!
//*****************************************************************************

void * distanceTask(void *arg0){


    Timer_Handle triggerTimer;
    Timer_Handle echoTimer;

    Timer_Params triggerParams;
    Timer_Params echoParams;

    t_RoverStatus roverStatus;
    TaskHandle_t roverTaskHandle;

    xQueueDebugSend("[DISTANCE TASK] Initiating task");

    /* Initialize timer */
    Timer_init();

    /* Configure parameters for trigger timer */
    /* This timer allows to control trigger pulse */
    Timer_Params_init(&triggerParams);
    triggerParams.period = TRIGGER_TIME ;
    triggerParams.periodUnits = Timer_PERIOD_US;
    triggerParams.timerMode =  Timer_ONESHOT_CALLBACK;
    triggerParams.timerCallback = prvTriggerTimerCallback;

    triggerTimer = Timer_open(Board_TIMER0, &triggerParams);
    configASSERT(triggerTimer);

    /* Configure parameters for echo timer */
    /* This timer let control if echo pulse  is bigger than security distance */
    Timer_Params_init(&echoParams);
    echoParams.period = ECHO_TIME ;
    echoParams.periodUnits = Timer_PERIOD_US;
    echoParams.timerMode =  Timer_ONESHOT_CALLBACK;
    echoParams.timerCallback = prvEchoTimerCallback;

    echoTimer = Timer_open(Board_TIMER1, &echoParams);
    configASSERT(triggerTimer);

    /* Configure call back function for echo pin interruptions */
    GPIO_setCallback(Board_GPIO_ECHO,prvEchoPinIntCallBack);

    /* Get rover task handle */
    roverTaskHandle = (TaskHandle_t) arg0;

    xQueueDebugSend("[DISTANCE TASK] Starting loop");

    while (1){

        /* Everything is going OK, clear the watch dog */
        watchdogReset();

        /* get rover current status */
        xQueueRoverStatusGet(&roverStatus);

        /* Check current rover status */
        /* and active necessary HC-SR04 sensor measurement */
        switch ( roverStatusGetDirection(roverStatus) ){
            /* Car is going forward, active front sensor */
            case FORWARD:
            case L_FORWARD:
            case R_FORWARD:

                /* Check distance [FRONT SENSOR] */
                activeSensor = 0;
                prvCheckDistance(triggerTimer, echoTimer, roverTaskHandle);
                break;

            /* Car is going backward, active back sensor, not active */
            case BACKWARD:
            case L_BACKWARD:
            case R_BACKWARD:

                /* Check distance [FRONT SENSOR]*/
                //activeSensor = 1;
                //prvCheckDistance(triggerTimer, echoTimer, roverTaskHandle, 1);
                break;
        }
        vTaskDelay(pdMS_TO_TICKS( 100 ));
    }

}

void prvCheckDistance(Timer_Handle triggerTimer, Timer_Handle echoTimer, TaskHandle_t roverTaskHandle){


    /* Create a counting semaphore */
    echoSemaphore= xSemaphoreCreateCounting(2,0);

    /* Create a binary semaphore */
    echoTimeoutSemaphore = xSemaphoreCreateBinary();

    /* active trigger pulse and interruptions */
    prvLaunchTriggerPulse(triggerTimer);

    /* Check for echo ping rise */
    if (pdPASS == xSemaphoreTake(echoSemaphore, pdMS_TO_TICKS( 1 ))){

        /* Start echo timeout timer */
        /* Echo pin status come from LOW to HIGH */
        if (Timer_start(echoTimer) == Timer_STATUS_ERROR) {
            configASSERT(0);
        };

        /* While time out is not coming */
        /* check if car is in security distance */
        while ( xSemaphoreTake(echoTimeoutSemaphore, 0) == pdFALSE ){

            /* Check for echo ping failing before timeout */
            /* Echo pin status come from HIGH to LOW */
            if (xSemaphoreTake(echoSemaphore,0) == pdPASS){

                /* Stop echo timer */
                Timer_stop(echoTimer);
                xQueueDebugSend("[DISTANCE TASK] In security distance, I will stop the car");

                /* Notify to rover control task about the obstacle in the road */
                xTaskNotifyGive(roverTaskHandle);

                /* Unlock echo timeout semaphore to escape from loop*/
                xSemaphoreGive(echoTimeoutSemaphore);
             }
         }

      }

    /* Free the semaphores */
    vSemaphoreDelete(echoTimeoutSemaphore);
    vSemaphoreDelete(echoSemaphore);
}

void prvLaunchTriggerPulse(Timer_Handle triggerTimer){

    /* Check sensor to activate */
    if (activeSensor == 0){
        /* Front sensor */
        GPIO_write(Board_GPIO_TRIG, Board_GPIO_PIN_HIGH);
    }
        /* Back sensor */
    else if (activeSensor == 1){
        /* To do */
    }

    /* Start trigger timer */
    if (Timer_start(triggerTimer) == Timer_STATUS_ERROR) {
        configASSERT(0);
            };
}


void prvTriggerTimerCallback(Timer_Handle myHandle){

    if (activeSensor == 0 ){
        /* Set trigger output to low */
        GPIO_write(Board_GPIO_TRIG, Board_GPIO_PIN_LOW);
    }
    /* Not in use */
    else if (activeSensor == 1){
        /* To Do */
    }

    /* Configure interruptions on echo port to detect low to high transition */
    GPIO_setConfig(Board_GPIO_ECHO, GPIO_CFG_IN_NOPULL | GPIO_CFG_IN_INT_RISING);

    /* Enable interruptions on Echo pin */
    GPIO_enableInt(Board_GPIO_ECHO);
}

void prvEchoTimerCallback(Timer_Handle myHandle){

    /* Unlock echo timeout semaphore*/
    xSemaphoreGiveFromISR(echoTimeoutSemaphore,NULL);

    /* Disable echo pin interruption */
    GPIO_disableInt(Board_GPIO_ECHO);


}

void prvEchoPinIntCallBack(uint_least8_t index){

    /* Check transition mode */
    if ( GPIO_read(Board_GPIO_ECHO) == 1 )
    {
        /* Change echo interruption mode to detect HIGH to LOW transition */
        GPIO_setConfig(Board_GPIO_ECHO, GPIO_CFG_IN_NOPULL | GPIO_CFG_IN_INT_FALLING);

        /* Unlock echo semaphore for first time */
        xSemaphoreGiveFromISR(echoSemaphore,NULL);
    }
    else if (GPIO_read(Board_GPIO_ECHO)==0){

        /* Unlock echo semaphore for second time */
        xSemaphoreGiveFromISR(echoSemaphore,NULL);

        /* Disable echo pin detection */
        GPIO_disableInt(Board_GPIO_ECHO);

    }
}
