/*
 * networktask.c
 *
 *  Created on: 15 abr. 2018
 *      Author: antonio
 */


#include "Network/networktask.h"
#include "FreeRTOS.h"
#include "task.h"

/* TI-DRIVERS Header files */
#include <ti/drivers/net/wifi/simplelink.h>

/* Queues in use */
#include <Queues/queuedebug.h>
#include <Queues/queuetelemetry.h>

#include <Telemetry/telemetry.h>
#include <Network/udpclient.h>
#include <Tda/networkstatus.h>
#include <Tda/networkstatus.h>
#include <Watchdog/watchdog.h>

#include "common_def.h"
#include "network_configuration.h"

#define CHANNEL_MASK_ALL            (0x1FFF)
#define RSSI_TH_MAX                 (-95)
#define SL_STOP_TIMEOUT             (200)

/* Local functions headers */

 //*************************************************************************
 //
 //! \brief This routine set default settings in the WNP
 //!
 //! \param  none
 //!
 //! \return none
 //!
 //*************************************************************************
void prvConfigureSimpleLinkToDefaultState(void);


 //*************************************************************************
 //
 //! \brief This routing set wireless configuration in the WNP
 //!
 //! \param  none
 //!
 //! \return none
 //!
 //*************************************************************************
 void prvWlanConnect(void);

 //*************************************************************************
 //
 //! \brief This routine set ip settings in the WNP
 //!
 //! \param  none
 //!
 //! \return none
 //!
 //*************************************************************************
void prvConfigureIPSettings(void);

 //*************************************************************************
 //
 //! \brief This routine active and configure the HTTP Server in the WNP
 //!
 //! \param  none
 //!
 //! \return none
 //!
 //*************************************************************************
 void prvConfigureHttpServer(void);

 //*************************************************************************
 //
 //! \brief This routine get a telemetry param  and
 //!        send it to the telemetry server
 //!
 //! \param  telemetry, telemetry structure to send to telemetry server
 //!
 //! \return none
 //!
 //*************************************************************************
 void prvProcessTelemetry(t_Telemetry telemetry);

 /******************************************************************************
 //!
 //! \brief Task responsible of synchronized network events
 //!         get telemetry and send it to telemtry server
 //!         reconfigure WNP settings if it is necessary
 //!
 //! \parm   arg, pointer to task arguments
 //!
 //! \return None
 //!
 //!*****************************************************************************
  */
void *networkTask(void *arg){

    _u16 len = sizeof(SlNetCfgIpV4Args_t);
    _u16 ConfigOpt = 0;
    SlNetCfgIpV4Args_t ipV4 = {0};

    t_Telemetry telemetry;
    t_Telemetry telemetry_old;
    BaseType_t xStatus;

    /* Initialize telemetry structures */
    telemetryInit(&telemetry);
    telemetryInit(&telemetry_old);

    /* Configure network processor default state */
    prvConfigureSimpleLinkToDefaultState();

    // Main loop
    while(1){

        /* Everything is going OK, clear the watch dog */
        watchdogReset();

        /* Check network status */
        switch ( xNetStatusGet() ){

            case NET_ERROR:

                /* Restart network */
                prvConfigureSimpleLinkToDefaultState();
                break;

            case NET_INIT:

                /* Configure wifi link */
                xQueueDebugSend("[NET TASK] Network initiated");
                prvWlanConnect();
                break;

            case HTTP_READY:

                /* HTTP SERVER READY FOR REMOTE CONTROL */

                /* Get telemetry from telemetry mail box */
                xStatus = xQueueTelemetryGet(&telemetry);

                /* Check if there a new telemetric values */
                if (xStatus==pdTRUE && telemetrycmp(telemetry, telemetry_old) != 0 ){
                    /* Send telemetry to the server */
                    prvProcessTelemetry(telemetry);
                    telemetrycpy(&telemetry_old, &telemetry);
                }

                break;

            case WLAN_DISCONNECTED:

                /* Try reconnect to the wifi link*/
                xQueueDebugSend("[NET TASK] Wireless LAN disconnected");
                prvWlanConnect();
                break;

            case WLAN_CONNECTED:
                /* Set ip settings */
                xQueueDebugSend("[NET TASK] Wireless LAN connected");
                prvConfigureIPSettings();
                break;

            case IP_ACQUIRED:

                /* Configure HTTP Server */
                xQueueDebugSend("[NET TASK] IP address acquired");

                /* Get NWP IP configuration */
                sl_NetCfgGet(SL_NETCFG_IPV4_STA_ADDR_MODE,&ConfigOpt,&len,(_u8 *)&ipV4);

                /* Check if NWP get ip settings */
                if ( ConfigOpt == SL_NETCFG_ADDR_DHCP ){

                    /* Reconfigure ip settings */
                    prvConfigureIPSettings();
                }
                else if ( ConfigOpt == SL_NETCFG_ADDR_STATIC){
                    /* Configure HTTP server */
                    prvConfigureHttpServer();
                }

                break;

            default:
                continue;
        }

        vTaskDelay(pdMS_TO_TICKS( 500 ));
    }

}

//*************************************************************************
 //
 //! \brief This routine get a telemetry param  and
 //!        send it to the telemetry server
 //!
 //! \param  xTelemetry
 //!
 //! \return none
 //!
 //*************************************************************************
void prvProcessTelemetry(t_Telemetry xTelemetry){


    char telemetry_string[MAX_STRING];

    xQueueDebugSend("[NET TASK] New telemetric dates");

    /* Process environment telemetry */
    telemetryEnvToString(xTelemetry,telemetry_string);
    vUDPClient(telemetry_string);
    *telemetry_string = '\0';

    /* Process gyroscope telemetry */
    telemetryGyroToString(xTelemetry,telemetry_string);
    vUDPClient(telemetry_string);
    *telemetry_string = '\0';

    /* Process accelerometer telemetry */
    telemetryAcceToString(xTelemetry,telemetry_string);
    vUDPClient(telemetry_string);
    *telemetry_string = '\0';

    /* Process compass telemetry */
    telemetryMagToString(xTelemetry,telemetry_string);
    vUDPClient(telemetry_string);
    *telemetry_string = '\0';

    /* Process rover status */
    telemetryRStatusToString(xTelemetry, telemetry_string);
    vUDPClient(telemetry_string);
    *telemetry_string = '\0';

}

//*************************************************************************
//
//! \brief This routine set default settings in the WNP
//!
//! \param  none
//!
//! \return none
//!
//*************************************************************************
void prvConfigureSimpleLinkToDefaultState(void)
 {
     uint8_t ucConfigOpt;
     uint8_t ucPower;
     int8_t RetVal = -1;
     uint32_t IfBitmap = 0;
     SlWlanScanParamCommand_t ScanDefault = {0};
     SlWlanRxFilterOperationCommandBuff_t RxFilterIdMask = {
         {0}
     };

     xQueueDebugSend("\r\n[NET TASK] Configuring link to default state \r\n");

     vNetStatusInit();

     RetVal = sl_Start(0, 0, 0);

     /* Set policy to auto only */
     RetVal =  sl_WlanPolicySet(SL_WLAN_POLICY_CONNECTION, SL_WLAN_CONNECTION_POLICY(1,0,0,0), NULL,0);
     configASSERT_ON_ERROR(RetVal);

     /* Disable Auto Provisioning */
     RetVal = sl_WlanProvisioning(SL_WLAN_PROVISIONING_CMD_STOP, 0xFF, 0, NULL, 0x0);
     configASSERT_ON_ERROR(RetVal);

     /* Delete existing profiles */
     RetVal = sl_WlanProfileDel(0xFF);
     configASSERT_ON_ERROR(RetVal);

     /* Disable ipv6 */
     IfBitmap = !(SL_NETCFG_IF_IPV6_STA_LOCAL | SL_NETCFG_IF_IPV6_STA_GLOBAL);
     RetVal = sl_NetCfgSet(SL_NETCFG_IF, SL_NETCFG_IF_STATE, sizeof(IfBitmap), (const unsigned char *)&IfBitmap);
     configASSERT_ON_ERROR(RetVal);

     /* Configure scan parameters to default */
     ScanDefault.ChannelsMask = CHANNEL_MASK_ALL;
     ScanDefault.RssiThreshold = RSSI_TH_MAX;

     RetVal = sl_WlanSet(SL_WLAN_CFG_GENERAL_PARAM_ID, SL_WLAN_GENERAL_PARAM_OPT_SCAN_PARAMS, sizeof(ScanDefault), (uint8_t *)&ScanDefault);
     configASSERT_ON_ERROR(RetVal);

     /* Disable scans */
     ucConfigOpt = SL_WLAN_SCAN_POLICY(0, 0);
     RetVal = sl_WlanPolicySet(SL_WLAN_POLICY_SCAN, ucConfigOpt, NULL, 0);
     configASSERT_ON_ERROR(RetVal);

     /* Set TX power lvl to max */
     ucPower = 0;
     RetVal = sl_WlanSet(SL_WLAN_CFG_GENERAL_PARAM_ID, SL_WLAN_GENERAL_PARAM_OPT_STA_TX_POWER,1, (uint8_t *)&ucPower);
     configASSERT_ON_ERROR(RetVal);

     /* Set NWP Power policy to 'normal' */
     RetVal = sl_WlanPolicySet(SL_WLAN_POLICY_PM, SL_WLAN_NORMAL_POLICY, NULL, 0);
     configASSERT_ON_ERROR(RetVal);

     /* Unregister mDNS services */
     RetVal = sl_NetAppMDNSUnRegisterService(0, 0, 0);
     configASSERT_ON_ERROR(RetVal);

     /* Remove all 64 RX filters (8*8) */
     memset(RxFilterIdMask.FilterBitmap, 0xFF, 8);

     RetVal = sl_WlanSet(SL_WLAN_RX_FILTERS_ID, SL_WLAN_RX_FILTER_REMOVE, sizeof(SlWlanRxFilterOperationCommandBuff_t), (uint8_t *)&RxFilterIdMask);
     configASSERT_ON_ERROR(RetVal);

     /* Set NWP role as STA */
     RetVal = sl_WlanSetMode(WIFI_ROLE);
     configASSERT_ON_ERROR(RetVal);

     /* Disable DHCP Server */
     RetVal = sl_NetAppStop(SL_NETAPP_DHCP_SERVER_ID);
     configASSERT_ON_ERROR(RetVal);

     /* For changes to take affect, we restart the NWP */
     RetVal = sl_Stop(SL_STOP_TIMEOUT);
     configASSERT_ON_ERROR(RetVal);

     RetVal = sl_Start(0, 0, 0);
     configASSERT_ON_ERROR(RetVal);

     // If everything is OK, then return OK
     xQueueDebugSend("[NET TASK] Default configuration ended");

 }

//*************************************************************************
//
//! \brief This routing set wireless configuration in the WNP
//!
//! \param  none
//!
//! \return none
//!
//*************************************************************************
 void prvWlanConnect(void){

     int8_t retc = 0;
      xQueueDebugSend("[NET TASK] Configuring link with the AP");

      if (WIFI_ROLE == ROLE_STA){

          /* Set NWP role as STA */
          retc = sl_WlanSetMode(WIFI_ROLE);
          configASSERT_ON_ERROR(retc);

          /* Configure params  */
          SlWlanSecParams_t secParams;
          secParams.Key = AP_SEC_PASSWORD;
          secParams.KeyLen = strlen(AP_SEC_PASSWORD);
          secParams.Type = AP_SEC_TYPE;

          /* For changes to take affect, we restart the NWP */
          retc = sl_Stop(SL_STOP_TIMEOUT);
          configASSERT_ON_ERROR(retc);

          retc = sl_Start(0, 0, 0);
          configASSERT_ON_ERROR(retc);

          /* Activate configuration */
          retc = sl_WlanConnect((const signed char *)(SSID), strlen((const char *)(SSID)), 0, &secParams, 0);
          configASSERT_ON_ERROR(retc);
      }
 }


 //*************************************************************************
 //
 //! \brief This routine active and configure the HTTP Server in the WNP
 //!
 //! \param  none
 //!
 //! \return none
 //!
 //*************************************************************************
void prvConfigureHttpServer(void){


       /* SSL Certificate */
       unsigned char ServerCertificateFileName[] = "/www/cert.pem";
       unsigned char ServerKeyFileName[] = "/www/privkey.pem";

       /* Authentication user and pass */
       unsigned char auth_name[] = PANEL_USER;
       unsigned char auth_pass[] = PANEL_PASS;

       /* HTTP Server params */
       unsigned char SecurityMode[] = {0x1};    // https mode
       unsigned char HttpsPort[] = {0xBB, 0x01}; // 0x1BB = 443
       unsigned char SecondaryPort[] = {0x50, 0x00}; // 0x050 = 80
       unsigned char SecondaryPortEnable[] = {0x1}; //

       unsigned char auth_enable = 1; // Enable web authentication
       unsigned char rom_access = 0;  // To block ROM HTTP ACCESS

       int8_t retc = 0;

       xQueueDebugSend("[NET_TASK] HTTP SERVER Configuration starting");

       retc = sl_NetAppStop(SL_NETAPP_HTTP_SERVER_ID);

       /* Set port */

       /*retc  = sl_NetAppSet (SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_PRIMARY_PORT_NUMBER, sizeof(SecondaryPort), SecondaryPort);
       configASSERT_ON_ERROR(retc); */

       /* Set the file names used for SSL key exchange. */
       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_DEVICE_CERTIFICATE_FILENAME, sizeof(ServerCertificateFileName),
                           ServerCertificateFileName);
       configASSERT_ON_ERROR(retc);

       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_PRIVATE_KEY_FILENAME, sizeof(ServerKeyFileName),
                           ServerKeyFileName);
       configASSERT_ON_ERROR(retc);

       /* Activate SSL security on primary HTTP port and change it to 443 (standard HTTPS port) */
       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_PRIMARY_PORT_SECURITY_MODE, sizeof(SecurityMode),
                           SecurityMode);
       configASSERT_ON_ERROR(retc);

       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_PRIMARY_PORT_NUMBER, sizeof(HttpsPort),
                           HttpsPort);
       configASSERT_ON_ERROR(retc);

       /* Enable secondary HTTP port (can only be used for redirecting connections to the secure primary port) */
       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_SECONDARY_PORT_NUMBER, sizeof(SecondaryPort),
                           SecondaryPort);
       configASSERT_ON_ERROR(retc);

       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_SECONDARY_PORT_ENABLE, sizeof(SecondaryPortEnable),
                           SecondaryPortEnable);
       configASSERT_ON_ERROR(retc);

       /* Set authentication */

       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_AUTH_CHECK, sizeof(auth_enable), &auth_enable);
       configASSERT_ON_ERROR(retc);

       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_AUTH_NAME, sizeof(auth_name), auth_name);
       configASSERT_ON_ERROR(retc);

       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID, SL_NETAPP_HTTP_AUTH_PASSWORD, sizeof(auth_pass), auth_pass);
       configASSERT_ON_ERROR(retc);

      // Block access to HTTP ROOT PAGES
       retc = sl_NetAppSet(SL_NETAPP_HTTP_SERVER_ID,SL_NETAPP_HTTP_ROM_PAGES_ACCESS, sizeof(rom_access), &rom_access);
       configASSERT_ON_ERROR(retc);

       // Restart
       retc = sl_NetAppStart(SL_NETAPP_HTTP_SERVER_ID);
       configASSERT_ON_ERROR(retc);

       xQueueDebugSend("[NET_TASK] HTTP SERVER Configuration finished");

       /* Set network status */
       vNetStatusSet(HTTP_READY);

   }


//*************************************************************************
//
//! \brief This routine set ip settings in the WNP
//!
//! \param  none
//!
//! \return none
//!
//*************************************************************************
void prvConfigureIPSettings(void){
    int8_t retc = 0;
    SlNetCfgIpV4Args_t net;

    xQueueDebugSend("[NET TASK] Setting fixed ip address");

    /* Fill SlNetCfgIPV4Args structure */
    net.Ip          = SL_IPV4_VAL(IP_A, IP_B, IP_C, IP_D);
    net.IpMask      = SL_IPV4_VAL(MASK_A, MASK_B, MASK_C, MASK_D);
    net.IpGateway   = SL_IPV4_VAL(GW_A, GW_B, GW_C, GW_D);
    net.IpDnsServer = SL_IPV4_VAL(DNS_A, DNS_B, DNS_C, DNS_D);

    /* Configure network settins on NP */
    retc = sl_NetCfgSet( SL_NETCFG_IPV4_STA_ADDR_MODE, SL_NETCFG_ADDR_STATIC,
                 sizeof(SlNetCfgIpV4Args_t), (unsigned char *)&net);
    configASSERT_ON_ERROR(retc);
}
