/*
 * l298n.h
 *
 *  Created on: 14 abr. 2018
 *      Author: antonio
 */

#ifndef DEBUG_L298N_H_
#define DEBUG_L298N_H_

#include "../Tda/roverdirection.h"
#include "FreeRTOS.h"

/******************************************************************************
//! Module to manage L298N drive circuit
//!
//! Correspondence between MSP432 Ports and L298N Circuit Port:
//!
//! L298N I1, I2, I3, and I4 pins are used to control the direction the motor will spin
//!
//! Board_GPIO_I1 P5.5 Output -> L298N PIN I1
//! Board_GPIO_I2 P5.4 Output -> L298N PIN I2
//! Board_GPIO_I3 P4.7 Output -> L298N PIN I3
//! Board_GPIO_I4 P4.5 Output -> L298N PIN I4
//!
//! L298N ENA, ENB pins are used to control the speed of the rotating motors.
//!
//! Board_PWM0 P2.7 -> L298N PIN ENA
//! Board_PWM1 P2.4 -> L288N PIN ENB
//!
//!*****************************************************************************
 */

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         only for clarify how connection to L298N works, all configurations
//!         are done at board initialization files
//!         (Board.h, MSP_EXP432P401R.c y MSP_EXP432P401R.h)
//!
//! \return None
//!
//!**********************************************************************************
 */
void l298nInit(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to rotate to the right side
//!
//! \return None
//!
//!**********************************************************************************
 */
void l298nRight(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to rotate to the left side
//!
//! \return None
//!
//!**********************************************************************************
 */
void l298nLeft(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to stop the vehicle
//!
//! \return None
//!
//!**********************************************************************************
 */
void l298nStop(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to move the vehicle forward
//!
//! \return None
//!
//!**********************************************************************************
 */
void l298nForward(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to move the vehicle to the right
//!
//! \return None
//!
//!**********************************************************************************
 */
void l298nForwardRight(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to move the vehicle to the left
//!
//! \return None
//!
//!**********************************************************************************
 */
void l298nForwardLeft(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to move the vehicle to the right (but going backward).
//!
//! \return None
//!
//!**********************************************************************************
*/
void l298nBackwardRight(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to move the vehicle to the left (but going backward).
//!
//! \return None
//!
//!**********************************************************************************
*/
void l298nBackwardLeft(void);

/************************************************************************************
//!
//! \brief  The routine configure GPIO ports connected to L298N pins (I1, I2, I3, I4)
//!         to move the vehicle backward
//!
//! \return None
//!
//!**********************************************************************************
*/
void l298nBackward(void);

/************************************************************************************
//!
//! \brief  The routine configure PWM ports to change vehicle speed
//!
//! \param  speed, a value between 1 and 10 to adjust PWM duty cycle
//!         10 is the maximum speed and 0 the minimum speed
//!
//! \return None
//!
//!**********************************************************************************
*/
void l298nSetSpeed(uint8_t speed);

/************************************************************************************
//!
//! \brief  The routine configure rover movement from a direction enum
//!
//! \param  direction, a value with direction to configure in the rover
//!
//! \return None
//!
//!**********************************************************************************
*/
void l298nSetDirection(direction_t direction);


#endif /* DEBUG_L298N_H_ */
