/*
 * rovertask.c
 *
 *  Created on: 15 abr. 2018
 *      Author: antonio
 */


#include <Queues/queueroverstatus.h>
#include <Queues/queuecommands.h>
#include <Queues/queuedebug.h>
#include <Queues/queuemode.h>

#include <Watchdog/watchdog.h>

#include "FreeRTOS.h"
#include "timers.h"
#include "semphr.h"
#include "task.h"

#include <Board.h>
#include <common_def.h>
#include <Rover/roverstatus.h>
#include <Rover/roverstatus.h>

#include <Tda/rovercommand.h>
#include <Tda/rovermode.h>


/* Local functions */

//*************************************************************************
//
//! \brief This timer call back mark the end of minimum time to execute a command
//!
//! \param  xTimer, pointer to originating timer
//!
//! \return none
//!
//*************************************************************************
void prvCommandTimerCallback( TimerHandle_t xTimer );

//*************************************************************************
//
//! \brief This routine configure a command in the L298N circuit
//!         and update rover status queue
//!
//! \param  xTimer, pointer to originating timer
//!
//! \return none
//!
//*************************************************************************
void prvExecCommand(t_RoverCommand roverCommand, t_RoverStatus *roverStatus);

//*************************************************************************
//
//! \brief This routine will execute command for given command execution time
//!
//! \param  roverStatus, pointer to a rover status structure
//!
//! \return none
//!
//*************************************************************************
void prvProcessCommand(t_RoverStatus *roverStatus);

//*************************************************************************
//
//! \brief This routine will execute sequence of commands in loop (LOOP MODE ACTIVATED)
//!
//!     First will wait for loops commands till receive a command with time=0 or
//!     when it reach the max commands limits
//!
//!     Second will execute commands in loops, the number of loops will be indicate
//!         by rover mode structure
//!
//! \param  roverStatus, pointer to a rover status structure
//!
//! \return none
//!
//*************************************************************************
void prvExecLoop(t_RoverStatus *roverStatus);


SemaphoreHandle_t commandLock;

TimerHandle_t commandTimer;


/* Main Task */

//*************************************************************************
//
//! \brief This task get commands from command queue and program L298N circuit
//!         to execute it
//!
//! \param  arg,    pointer to task arguments
//!
//! \return none
//!
//*************************************************************************

void *roverTask(void *arg){

    t_RoverStatus roverStatus;
    t_Mode xRoverMode;


     xQueueDebugSend("[ROVER TASK] It will initialize rover status");

     /* Initiating rover status */
     roverStatusInit(&roverStatus);
     xQueueRoverStatusSet(&roverStatus);

     /* Initiating rover mode */
     vRoverModeInit(&xRoverMode);
     vQueueModeSet(&xRoverMode);

     /* Create command timer */
     commandTimer = xTimerCreate("COMMAND_TIMER", pdMS_TO_TICKS(1000),
                                 pdFALSE, 0, prvCommandTimerCallback);
     configASSERT(commandTimer);


    /* Start task loop  */
    xQueueDebugSend("[ROVER TASK] Starting loop");

    while (1){
        
        /* Everything is going OK, clear the watch dog */
        watchdogReset();

        /* Get rover mode */
        vQueueModeGet(&xRoverMode);

       if ( ucRoverModeGetMode(xRoverMode) == MODE_INTERACTIVE){
           /* Mode interactive */
           prvProcessCommand(&roverStatus);
       }
           /* Mode loop */
       else {
           prvExecLoop(&roverStatus);
       }

       vTaskDelay(pdMS_TO_TICKS( 50 ));

    }

}

void prvExecLoop(t_RoverStatus *roverStatus){

    t_Mode xRoverMode;
    t_RoverCommand xRoverCommand[MAX_LOOP_COMMANDS];
    uint8_t ucCommandCounter = 0;
    uint8_t ucLoops = 0;
    uint8_t ucCounter;

    /* Get rover mode */
    vQueueModeGet(&xRoverMode);

    xQueueDebugSend("[ROVER TASK] Mode Loop - Reading commands from Queue");

    /* Read commands from queue till MAX_LOOP_COMMANDS or end sequence command */
    while (ucCommandCounter < MAX_LOOP_COMMANDS){

        /* Get rover command from queue */
        if (xQueueCommandsReceive(&xRoverCommand[ucCommandCounter]) != NULL){

            /* Check end of sequence command (command time equals to zero) */
            if (roverCommandGetTime(xRoverCommand[ucCommandCounter]) == 0){
                break;
            }
            else {
                /* Increase counter */
                ucCommandCounter++;
            }
        }
    }

    /* Get number of loops */
    ucLoops = ucRoverModeGetLoop(xRoverMode);

    /* Clear queue commands */
    vQueueCommandReset();

    /* Exec Commands from queue */
    xQueueDebugSend("[ROVER TASK] End commands reading. Starting loops");

    /* Exec loops */
    while ( ucLoops > 0){
        /* Exec sequence of commands */
        for (ucCounter = 0;  ucCounter < ucCommandCounter; ucCounter++){
            /* Send each command to the queue */
            xQueueCommandsSend(&xRoverCommand[ucCounter]);
            /* Process command from queue */
            prvProcessCommand(roverStatus);
        }
        ucLoops --;

    }

    /* Reset mode */
    /* It will go to interactive mode again */
    vRoverModeInit(&xRoverMode);
    vQueueModeSet(&xRoverMode);


    xQueueDebugSend("[ROVER TASK] Ending loop");

}

void prvCommandTimerCallback( TimerHandle_t xTimer ){

    uint8_t ucRetc;
    /* Unlock the command semaphore */
    xQueueDebugSendFromISR("[ROVER TASK] TimerEnded");
    ucRetc=xSemaphoreGiveFromISR( commandLock, NULL );
    configASSERT(ucRetc);
}

void prvExecCommand(t_RoverCommand roverCommand, t_RoverStatus *roverStatus){

    /* Set rover direction and speed */
    roverStatusSetSpeed(roverStatus, roverCommand.speed);
    roverStatusSetDirection(roverStatus, roverCommand.direction);
    /* Update rover status queue */
    xQueueRoverStatusSet(roverStatus);
}


void prvProcessCommand(t_RoverStatus *roverStatus){

    t_RoverCommand roverCommand;
     uint8_t retc;

    /* Check command queue */
      if ( xQueueCommandsReceive(&roverCommand) != NULL){

          xQueueDebugSend("[ROVER TASK] It get a new command from network");

          /* Set rover direction and speed */
          prvExecCommand(roverCommand, roverStatus);

          // Check if command has a fixed execution time
          if (roverCommandGetTime(roverCommand) > 0){

              xQueueDebugSend("[ROVER TASK] Programmed command");

              /* Create a binary semaphore */
             commandLock = xSemaphoreCreateBinary();
             configASSERT(commandLock);

             /* Configure timer for command execution time */
             retc = xTimerChangePeriod( commandTimer, pdMS_TO_TICKS(roverCommandGetTime(roverCommand)*1000) , 0);
             configASSERT(retc);

             /* Start the timer */
              retc = xTimerStart( commandTimer, 0 );
              configASSERT(retc);

              /* Wait for end of timer */
              while ( xSemaphoreTake( commandLock, 0) == pdFALSE){
                  /* Check for obstacle notification from distance task */
                  if (ulTaskNotifyTake( pdTRUE, 0 ) > 0){
                      roverStatusSetDirection(roverStatus, LOCK_FORWARD);
                      xQueueRoverStatusSet(roverStatus);
                  }
                  vTaskDelay(pdMS_TO_TICKS( 50 ));
              }

              /* Delete a binary semaphore */
              vSemaphoreDelete( commandLock );
              /* Stop the timer */
              retc = xTimerStop( commandTimer, 0);
          }

          /* Check for obstacle notification from distance task */
            if (ulTaskNotifyTake( pdTRUE, 0 ) > 0){
                roverStatusSetDirection(roverStatus, LOCK_FORWARD);
                xQueueRoverStatusSet(roverStatus);
            }

      }


}



