/*
 * Copyright (c) 2016, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ======== main_freertos.c ========
 */
#include <stdint.h>
#include <stdlib.h>


/* RTOS header files */
#include "FreeRTOS.h"
#include "task.h"

/* TI-RTOS Header files */
#include <ti/drivers/GPIO.h>
#include <ti/drivers/net/wifi/simplelink.h>

/* Example/Board Header files */
#include "Board.h"

#include <Queues/queuedebug.h>
#include <Queues/queuetelemetry.h>
#include <Queues/queueroverstatus.h>
#include <Queues/queuecommands.h>
#include <Queues/queuemode.h>

/* Stack size in bytes */

#define NETWORK_STACKSIZE  1024
#define CONTROL_STACKSIZE  1024
#define DEBUG_STACKSIZE    512
#define DISTANCE_STACKSIZE 512
#define WATCHDOG_STACKSIZE 256
#define SL_STACK_SIZE      1024

extern void * debugTask(void *arg0);
extern void * networkTask(void *arg0);
extern void * roverTask(void *arg0);
extern void * telemetryTask(void *arg);
extern void * distanceTask(void *arg0);
extern void * watchdogTask(void *arg0);


/* Task handles */

TaskHandle_t debugHandle;
TaskHandle_t networkHandle;
TaskHandle_t roverHandle;
TaskHandle_t telemetryHandle;
TaskHandle_t distanceHandle;
TaskHandle_t wdgHandle;
TaskHandle_t slHandle;

/*
 *  ======== main ========
 */
int main(void)
{

    BaseType_t   xRetc;

    /* Call driver init functions */
    Board_initGeneral();
    Board_initSPI();
    Board_initGPIO();

    /* Initiating queues */
    vQueueDebugInit();
    vQueueTelemetryInit();
    vQueueRoverStatusInit();
    vQueueCommandsInit();
    vQueueModeInit();

    xQueueDebugSend("[MAIN TASK] Creating tasks");

    /*
     * Create debug task
     */

    xRetc = xTaskCreate((TaskFunction_t)debugTask,     // pvTaskCode
                               NULL,                  // pcName
                               DEBUG_STACKSIZE,       // usStackDepth
                               NULL,                  // pvParameters
                               1,                     // uxPriority
                               &debugHandle);         // pxCreatedTask
    configASSERT(xRetc);

    /*
     * Create network task
     */

    xRetc = xTaskCreate((TaskFunction_t)networkTask,     // pvTaskCode
                                NULL,                   // pcName
                                NETWORK_STACKSIZE,      // usStackDepth
                                NULL,                   // pvParameters
                                2,                      // uxPriority
                                &networkHandle);        // pxCreatedTask
    configASSERT(xRetc);

    /*
     * Create rover task
     */
    xRetc = xTaskCreate((TaskFunction_t)roverTask,     // pvTaskCode
                                NULL,                 // pcName
                                CONTROL_STACKSIZE,    // usStackDepth
                                NULL,                 // pvParameters
                                3,                    // uxPriority
                                &roverHandle);        // pxCreatedTask
    configASSERT(xRetc);


    /*
     * Create network task
     */

    xRetc = xTaskCreate((TaskFunction_t) telemetryTask,    // pvTaskCode
                                NULL,                     // pcName
                                NETWORK_STACKSIZE,        // usStackDepth
                                NULL,                     // pvParameters
                                1,                        // uxPriority
                                &telemetryHandle);        // pxCreatedTask
    configASSERT(xRetc);


    /*
     * Create anti-collision task (distance task)
     * pass the rover handle like a parameter
     * this will allow send notification to the rover task
     */


    xRetc = xTaskCreate((TaskFunction_t) distanceTask,    // pvTaskCode
                                NULL,                    // pcName
                                DISTANCE_STACKSIZE,      // usStackDepth
                                (void *) roverHandle ,   // pvParameters - Pass rover task handle to the task to allow notifications
                                4,                       // uxPriority
                                &distanceHandle);        // pxCreatedTask
    configASSERT(xRetc);

    /*
     * Create watch dog task (distance task)
     */


    xRetc = xTaskCreate((TaskFunction_t) watchdogTask,    // pvTaskCode
                                NULL,                    // pcName
                                WATCHDOG_STACKSIZE,      // usStackDepth
                                NULL,                    // pvParameters
                                8,                       // uxPriority
                                &wdgHandle);             // pxCreatedTask
    configASSERT(xRetc);

    /*
     * The SimpleLink host driver architecture mandate spawn thread
     * to be created prior to calling Sl_start (turning the NWP on).
     * The purpose of this thread is to handle asynchronous events sent from the NWP.
     * Every event is classified and later handled by the Host driver event handlers.
     */

    xRetc = xTaskCreate((TaskFunction_t)sl_Task,        // pvTaskCode
                                NULL,                  // pcName
                                SL_STACK_SIZE,         // usStackDepth
                                NULL,                  // pvParameters
                                10,                    // uxPriority
                                &slHandle);            // pxCreatedTask
    configASSERT(xRetc);


    /* Start the FreeRTOS scheduler */
    xQueueDebugSend("[MAIN TASK] Starting scheduler");
    vTaskStartScheduler();

    return (0);
}

//*****************************************************************************
//
//! \brief Application defined malloc failed hook
//!
//! \param  none
//!
//! \return none
//!
//*****************************************************************************
void vApplicationMallocFailedHook()
{
    /* Handle Memory Allocation Errors */
    while(1)
    {
    }
}

//*****************************************************************************
//
//! \brief Application defined stack overflow hook
//!
//! \param  none
//!
//! \return none
//!
//*****************************************************************************
void vApplicationStackOverflowHook(TaskHandle_t pxTask,
                                   char *pcTaskName)
{
    //Handle FreeRTOS Stack Overflow
    while(1)
    {
    }
}

void vApplicationTickHook(void)
{
    /*
     * This function will be called by each tick interrupt if
     * configUSE_TICK_HOOK is set to 1 in FreeRTOSConfig.h.  User code can be
     * added here, but the tick hook is called from an interrupt context, so
     * code must not attempt to block, and only the interrupt safe FreeRTOS API
     * functions can be used (those that end in FromISR()).
     */
}

void vPreSleepProcessing(uint32_t ulExpectedIdleTime)
{
}

//*****************************************************************************
//
//! \brief Application defined idle task hook
//!
//! \param  none
//!
//! \return none
//!
//*****************************************************************************
void
vApplicationIdleHook(void)
{
    /* Handle Idle Hook for Profiling, Power Management etc */
}

//*****************************************************************************
//
//! \brief  Overwrite the GCC _sbrk function which check the heap limit related
//!         to the stack pointer.
//!			In case of freertos this checking will fail.
//! \param  none
//!
//! \return none
//!
//*****************************************************************************
#if defined (__GNUC__)
void * _sbrk(uint32_t delta)
{
    extern char _end;     /* Defined by the linker */
    extern char __HeapLimit;
    static char *heap_end;
    static char *heap_limit;
    char *prev_heap_end;

    if(heap_end == 0)
    {
        heap_end = &_end;
        heap_limit = &__HeapLimit;
    }

    prev_heap_end = heap_end;
    if(prev_heap_end + delta > heap_limit)
    {
        return((void *) -1L);
    }
    heap_end += delta;
    return((void *) prev_heap_end);
}

#endif
